/** Macro to disallow copy and assigment. Use in `private` class section (so
 * external code does not compile.) It declares, but does not provide any
 * corresponding definition (so that any internal attempt to use them results in
 * a link error.)*/
#define DISALLOW_COPY_AND_ASSIGN(T) \
   void T(const T&);                \
   void operator=(const T&)
/** Makes sub-classes non-copyable by using @ref DISALLOW_COPY_AND_ASSIGN. @see
 * <a href="http://www.boost.org/doc/libs/1_40_0/libs/utility/utility.htm#Class_
 * noncopyable"
 * title="Header boost/utility.hpp Documentation - 1.40.0">Boost utilities</a>*/
class Noncopyable{
 protected:                                                       // Abstract
   void     Noncopyable(void){}                                         //[Ctor]
 /// @cond Skip
 private:   DISALLOW_COPY_AND_ASSIGN(Noncopyable);
 /// @endcond
};
//+- Noncopyable ----------------------------------------------------+
class BrokerNameingPattern : public Noncopyable{
   /* isolate prefix, infix, and postfix extra characters. Broker's use a
    * variety of nameing patterns: EURUSD, EURUSDm, EURUSDi "EURUSD.",
    * "EURUSD..", "EUR.USD", "EUR/USD", "EURUSD.stp", EURUSDct, "EURUSD.G",
    * "EURUSD+", EURUSDpro at least.                                          */
 public:
   bool     init(void){                                        // Delayed Ctor
      COUNT    nSymbols = SymbolsTotal(false);
      if(nSymbols < 2){
         Alert("SymbolsTotal(false) returns zero Err:", _LastError);
         return false;  }
      int   testChar = 0;
      //{ Find first column that has changing characters. That starts the base.
      const INDEX iPREFIX  = 0;              mIndexBase  = iPREFIX - 1;
      for(INDEX iChart = 0; true; iChart = (iChart+1) % nSymbols){
         string   symbol   = SymbolName(iChart, false);
         if(      iChart == 0)
                  testChar = StringGetChar(symbol, ++mIndexBase);
         else  if(testChar != StringGetChar(symbol, mIndexBase))  break;
      }
      //} Find first column that has changing characters. That starts the base.
      mPrefix  = mIndexBase == iPREFIX
               ? "" : StringSubstr(_Symbol, iPREFIX, mIndexBase - iPREFIX);
      //{ Find next column that has changing characters. That starts the quote.
      INDEX    iInfix   = mIndexBase + 3;    mIndexQuote = iInfix - 1;
      for(INDEX iChart = 0; true; iChart = (iChart+1) % nSymbols){
         string   symbol   = SymbolName(iChart, false);
         if(      iChart == 0)
                  testChar = StringGetChar(symbol, ++mIndexQuote);
         else  if(testChar != StringGetChar(symbol, mIndexQuote)) break;
      }
      //} Find next column that has changing characters. That starts the quote.
      mInfix   = mIndexQuote == iInfix
               ? "" : StringSubstr(_Symbol, iInfix,  mIndexQuote - iInfix);
      INDEX    iPosfix  = mIndexQuote + 3;   COUNT nChar = StringLen(_Symbol);
      mPostfix = nChar == iPosfix
               ? "" : StringSubstr(_Symbol, iPosfix, nChar - iPosfix);
      return true;
   }  // init
 public:                                                          // Methods
   string   base(const SYMBOL& symbol){
                                    return StringSubstr(symbol, mIndexBase, 3);}
   string   quote(const SYMBOL& symbol){
                                    return StringSubstr(symbol, mIndexQuote,3);}
   SYMBOL   symbol(const string& base, const string& quote){
      return StringConcatenate(mPrefix, base, mInfix, quote, mPostfix);
   }
 private:                                                         // Data
   INDEX    mIndexBase, mIndexQuote;
   string   mPrefix,    mInfix,     mPostfix;
};
//+- BrokerNameingPattern -------------------------------------------+
